# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
$LOAD_PATH.unshift File.join(File.dirname(__FILE__), '../..')
require 'puppet_x/enterprisemodules/weblogic/core'

module Puppet
  Type.newtype(:wls_workmanager_constraint) do
    include Puppet_X::EnterpriseModules::WebLogic::Type

    desc <<-DESC
    This resource allows you to manage work manager constraints in an WebLogic domain.

    Here is an example on how you should use this:

        wls_workmanager_constraint { 'CapacityConstraint':
          ensure          => 'present',
          constrainttype  => 'Capacity',
          constraintvalue => '20',
          target          => ['WebCluster'],
          targettype      => ['Cluster'],
        }

    In this example you are managing a work manager contraint in the default domain. When you want to
    manage a work manager contraint in a specific domain, you can use:

        wls_workmanager_constraint { 'my_domain/MaxThreadsConstraint':
          ensure          => 'present',
          constrainttype  => 'MaxThreadsConstraint',
          constraintvalue => '5',
          target          => ['WebCluster'],
          targettype      => ['Cluster'],
        }

    Here are some more examples:

        wls_workmanager_constraint { 'MinThreadsConstraint':
          ensure          => 'present',
          constrainttype  => 'MinThreadsConstraint',
          constraintvalue => '2',
          target          => ['WebCluster'],
          targettype      => ['Cluster'],
        }
        wls_workmanager_constraint { 'FairShareReqClass':
          ensure          => 'present',
          constrainttype  => 'FairShareRequestClasses',
          constraintvalue => '50',
          target          => ['WebCluster'],
          targettype      => ['Cluster'],
        }

    DESC

    #
    # Only use elements on the path where this condition is true. The condition is
    # that is has specified serverTemplates
    #
    # For now exclude ContextRequestClasses. They don't match the predefined API of the type
    # If we need to, we can create a new type for that.
    #
    index_condition do |_name, path, _wls_config|
      !%w[ContextRequestClasses WorkManagers].include?(path.split('/').last)
    end

    bean_path('/SelfTuning/%<domain_name>s/%<constrainttype_folder>s')

    on_create do
      Puppet.info "create #{name}"
      actual_creation_path = bean_path.subst(to_hash)
      fail "Parent resource #{actual_creation_path} doesn't exists." unless wls_config.exists?(actual_creation_path)

      wls_config.create(constrainttype, actual_creation_path, self[key_name])
      nil
    end

    on_destroy do
      Puppet.info "destroy #{name} "
      full_path = bean_path.subst(to_hash).to_s
      wls_config.destroy(provider.constrainttype, full_path, self[key_name])
    end

    #
    # Because bean_path uses the variable constrainttype_folder, we need to have it available all the time
    # This code adds it to the hash based on the setting of constrainttype.
    #
    def to_hash
      hash = super
      hash[:constrainttype_folder] = constrainttype_to_folder
      key = case hash[:constrainttype_folder]
            when 'FairShareRequestClasses' then 'FairShare'
            when 'ResponseTimeRequestClasses' then 'GoalMs'
            else
              'Count'
            end
      hash[:value] = key
      hash
    end

    ensurable
    parameter :domain
    parameter :name
    parameter :workmanager_constraint_name
    parameter :timeout

    property :constrainttype
    property :target
    property :targettype
    property :constraintvalue

    add_title_attributes(:workmanager_constraint_name)

    private

    def constrainttype_to_folder
      working_value = provider.constrainttype.to_s == 'absent' ? constrainttype : provider.constrainttype
      case working_value.to_s
      when 'MaxThreadsConstraint' then 'MaxThreadsConstraints'
      when 'MinThreadsConstraint' then 'MinThreadsConstraints'
      when 'Capacity' then 'Capacities'
      when 'FairShareRequestClass' then 'FairShareRequestClasses'
      when 'ResponseTimeRequestClass' then 'ResponseTimeRequestClasses'
      else
        fail "#{working_value} is an invalid constrainttype"
      end
    end
  end
end
